/* This source file is part of the ATMEL AVR32-SoftwareFramework-AT32UC3B-1.4.0 Release */

//_____  I N C L U D E S ___________________________________________________

#include "compiler.h"
#include "preprocessor.h"
#include "board.h"
#include "ctrl_access.h"
#include "pm.h"
#include "gpio.h"
#include "spi.h"
#include "conf_sd_mmc_spi.h"
#include "sd_mmc_spi.h"
#include "flashc.h"
#include "fat.h"
#include "file.h"
#include "navigation.h"
#include "fsaccess.h"


//_____ M A C R O S ________________________________________________________

//! \brief PBA clock frequency (Hz)
#define PBA_HZ                24000000	//FOSC0

//! string for first arg
static unsigned char READBUFFER[0x4800+8];

static int max_avilen;

extern void init_oled_disp(void);
extern void disp_off(void);
extern void disp_data(unsigned char *buf);

// internal clock=48MHz,PBA=24MHz,PBB=48MHz
void local_start_pll0(volatile avr32_pm_t* pm)
{
  pm_switch_to_osc0(pm, FOSC0, OSC0_STARTUP);  // Switch main clock to Osc0.
  pm_pll_setup(pm,
               0,   // use PLL0
               7,   // MUL=7 in the formula
               1,   // DIV=1 in the formula
               0,   // Sel Osc0/PLL0 or Osc1/PLL1
               16); // lockcount in main clock for the PLL wait lock
  pm_pll_set_option(pm, 0, 1, 1, 0);
  pm_pll_enable(pm,0);
  pm_wait_for_pll0_locked(pm) ;
  pm_cksel(pm, 1, 0, 0, 0, 0, 0);
  flashc_set_wait_state(1);
  pm_switch_to_clock(pm, AVR32_PM_MCSEL_PLL0); /* Switch main clock to 48MHz */
}

/*! \brief Initializes SD/MMC resources: GPIO, SPI and SD/MMC.
 */
static void sd_mmc_resources_init(void)
{
  // GPIO pins used for SD/MMC interface
  static const gpio_map_t SD_MMC_SPI_GPIO_MAP =
  {
    {SD_MMC_SPI_SCK_PIN,  SD_MMC_SPI_SCK_FUNCTION },  // SPI Clock.
    {SD_MMC_SPI_MISO_PIN, SD_MMC_SPI_MISO_FUNCTION},  // MISO.
    {SD_MMC_SPI_MOSI_PIN, SD_MMC_SPI_MOSI_FUNCTION},  // MOSI.
    {SD_MMC_SPI_NPCS_PIN, SD_MMC_SPI_NPCS_FUNCTION},   // Chip Select NPCS.
    {AVR32_SPI_NPCS_2_0_PIN, AVR32_SPI_NPCS_2_0_FUNCTION} // Chip Select NPCS
  };

  // SPI options.
  spi_options_t spiOptions =
  {
    .reg          = SD_MMC_SPI_NPCS,
    .baudrate     = SD_MMC_SPI_MASTER_SPEED,  // Defined in conf_sd_mmc.h.
    .bits         = SD_MMC_SPI_BITS,          // Defined in conf_sd_mmc.h.
    .spck_delay   = 0,
    .trans_delay  = 0,
    .stay_act     = 1,
    .spi_mode     = 0,
    .modfdis      = 1
  };

  static const spi_options_t SPI_OPTIONS_OLED =
  {
    .reg          = 2,
    .baudrate     = 10000000,  // 10MHz
    .bits         = 8,
    .spck_delay   = 0,
    .trans_delay  = 0,
    .stay_act     = 1,
    .spi_mode     = 0,
    .modfdis      = 1
  };

  spi_reset(&AVR32_SPI);  // spi reset

  // Assign I/Os to SPI.
  gpio_enable_module(SD_MMC_SPI_GPIO_MAP,
                     sizeof(SD_MMC_SPI_GPIO_MAP) / sizeof(SD_MMC_SPI_GPIO_MAP[0]));

  // Initialize as master.
  spi_initMaster(SD_MMC_SPI, &spiOptions);
  spi_initMaster(&AVR32_SPI, &SPI_OPTIONS_OLED);

  // Set SPI selection mode: variable_ps, pcs_decode, delay.
  spi_selectionMode(SD_MMC_SPI, 0, 0, 0);

  // Enable SPI module.
  spi_enable(SD_MMC_SPI);

  spi_setupChipReg(&AVR32_SPI,&SPI_OPTIONS_OLED,PBA_HZ);
  // Initialize SD/MMC driver with SPI clock (PBA).
  sd_mmc_spi_init(spiOptions, PBA_HZ);
}

// goto LISTmovi chunk
static int skip_chunk(void)
{
static const char RIFFstr[]="RIFF";
static const char AVIstr[]="AVI ";
static const char LISTstr[]="LIST";
static const char movistr[]="movi";
	int i;
	unsigned long chunksize;

    file_seek(0,FS_SEEK_SET);
	file_read_buf(READBUFFER,8);	// RIFF
	for (i=0;i<4;i++)
	  if (READBUFFER[i]!=RIFFstr[i]) return -1;
	max_avilen=Swap32(*(unsigned long *)&READBUFFER[4]);
	file_read_buf(READBUFFER,4);	// AVI
	for (i=0;i<4;i++)
	  if (READBUFFER[i]!=AVIstr[i]) return -1;
	while (1) {
	  if (file_read_buf(READBUFFER,4)!=4) return -1;
	  for (i=0;i<4;i++)
	    if (READBUFFER[i]!=LISTstr[i]) break;
	  if (file_read_buf(READBUFFER,4)!=4) return -1;
	  chunksize=Swap32(*(unsigned long *)&READBUFFER[0]);
	  if (i==4) {
	    // read sub chunk name
	    if (file_read_buf(READBUFFER,4)!=4) return -1;
	    for (i=0;i<4;i++)
	      if (READBUFFER[i]!=movistr[i]) break;
	    if (i==4) return chunksize; // found LISTmovi chunk
	    if (chunksize<4) return -1;
	    chunksize-=4;
	  }
	  if (!file_seek(chunksize,FS_SEEK_CUR_FW)) return -1; // next list chunk
	}
}

// disp
static void disp_movie(int endpos)
{
static const char chunkstr[]="00db"; // no-compression video frame
	int res,i,isskipframe;
	unsigned long nowpos,framesize;

	nowpos=0;
	while (nowpos<endpos) {
	  res=file_read_buf(READBUFFER,8);
	  nowpos+=res;
	  if (res==0x8) {
		isskipframe=0;
	    for (i=0;i<4;i++)
	      if (READBUFFER[i]!=chunkstr[i]) isskipframe=1;
	    framesize=Swap32(*(unsigned long*)&READBUFFER[4]);
	    if (framesize!=0x4800) isskipframe=1; // 96*64*3byte
	    if (isskipframe) {
	      file_seek(framesize,FS_SEEK_CUR_FW);
	      nowpos+=framesize;
	      continue;
	    }
	    res=file_read_buf(&READBUFFER[0],0x4800);
	    if (res!=0x4800) return;
	    nowpos+=res;
	  } else
	    break; // eof or dust data.
	  disp_data(&READBUFFER[0]);
      if (!gpio_get_pin_value(GPIO_PUSH_BUTTON_0)) {
   	     while (gpio_get_pin_value(GPIO_PUSH_BUTTON_0)) ; // wait push button0
   	     break; // wait push button0
      }
	}
}

/*! \brief Main function. Execution starts here.
 */
int main(void)
{
  int chunksize;

  local_start_pll0(&AVR32_PM);

  sd_mmc_resources_init();
  init_oled_disp();

  // initialize FSACCESS mutex and navigators
  b_fsaccess_init();
  nav_reset();

  // Select the desired drive.
  nav_drive_set(0);

  // Try to mount it.
  if (nav_partition_mount())
  {
     while (gpio_get_pin_value(GPIO_PUSH_BUTTON_0)) ; // wait push button0
     while (!gpio_get_pin_value(GPIO_PUSH_BUTTON_0)) ; // wait push button0
     if (nav_setcwd((FS_STRING)"A:/TEST.AVI\0", FALSE, FALSE)) {
       if (file_open(FOPEN_MODE_R)) {
		 chunksize=skip_chunk();
		 if (chunksize>0) disp_movie(chunksize);
	     file_close();
       }
     }
     while (gpio_get_pin_value(GPIO_PUSH_BUTTON_0)) ; // wait push button0
  }
  disp_off();

  return 0;
}
